#include <iostream>
using namespace std;

#include "vdop.h"

void vddiv(const int n,  float x[], const int ix,
           const float y[], const int iy,
           const float z[], const int iz)
{
    #pragma omp parallel for default(none) shared(x,y,z)
    for (int k = 0; k < n; ++k) {
        x[k * ix] = y[k * iy] / z[k * iz];
    }
    return;
}



void vdaxpy(const int n, float x[], const float y[],
            const float a,   const float z[] )
{
#pragma omp parallel for default(none) shared(x,y,z)
#pragma acc kernels loop independent pcopyout(x[0:n]) pcopyin(y[0:n],z[0:n])
    for (int k = 0; k < n; ++k) {
        x[k] = y[k] + a * z[k];
    }
    return;
}



/******************************************************************************/
#ifdef _OPENACC
//#include <accel.h>              // OpenACC

float dscapr(const int n, const float x[], const float y[])
{
    const int NUM_SMX = 2,
              NUM_THREADS = 256;
              //NUM_THREADS = 1024;
    float    s = 0.0;

#pragma omp parallel for default(none) shared(x,y) reduction(+:s)
#pragma acc parallel num_gangs(NUM_SMX) vector_length(NUM_THREADS) \
                            pcopyin(x[0:n],y[0:n])
    {
#pragma acc loop reduction(+:s)
        for (int thread = 0; thread < NUM_SMX * NUM_THREADS; ++thread) {
            float st = 0.0;
#pragma acc loop seq
            for (int k = thread; k < n; k += NUM_SMX * NUM_THREADS) {
                st += x[k] * y[k];
            }
            s +=st;
        }
    }
    return s;
}

#else
float dscapr(const int n, const float x[], const float y[])
{
    float    s = 0.0;

#pragma omp parallel for default(none) shared(x,y) reduction(+:s)
#pragma acc kernels loop pcopyin(x[0:n],y[0:n]) reduction(+:s)
    for (int k = 0; k < n; ++k) {
        s += x[k] * y[k];
    }

    return s;
}

#endif

