#ifndef HEADER_VECACCU
#define HEADER_VECACCU
#include <mpi.h>

/**
 * Accumulates vector @p w, i.e., a distributed vector is converted into
 * an accumulated vector.
 * Dynamic memory allocation in the first call of this function.
 * @param[in] nx	number of discretization intervals in x-direction
 * @param[in] ny	number of discretization intervals in y-direction
 * @param[in,out] w	vector for all nodes of local discretization
 * @param[in] neigh	vector of 4 elements containing the ranks of neighbouring processes, -1 indicates a boundary in this direction
 * @param[in] color	red/black coloring of subdomains (0/1)
 * @param[in] icomm	communicator
*/

void VecAccu(const int nx, const int ny, float w[],
	const int neigh[], const int color, const MPI::Intracomm& icomm);
/**
 * Frees the dynamic memory allocated in the first call in VecAccu.
 * @see VecAccu
*/
void FreeVecAccu();

#endif
