
#include <iostream>
#include <cmath>
using namespace std;

//----------------------------------------------------------------------
//    from  CUDA/skalar/skalar_3.cu
//
//    kernel function on device
__global__ void scalar(float *sg, const float *x, const float *y, int N)
{
    extern __shared__ float sdata[];
    const int idx = blockIdx.x * blockDim.x + threadIdx.x;
    const int str = gridDim.x * blockDim.x;

    float s = 0.0;
    for (int i = idx; i < N; i += str)
        s += x[i] * y[i];
    sdata[threadIdx.x] = s;

    __syncthreads();
    for (int s = blockDim.x >> 1; s > 0; s >>= 1) {
        if (threadIdx.x < s) {
            sdata[threadIdx.x] += sdata[threadIdx.x + s];
        }
        __syncthreads();
    }
    if (threadIdx.x == 0)  sg[blockIdx.x] = sdata[0];
}


//    kernel function for addition on  o n e  block on device
__global__ void add_1_block(float *s, int N)
{
    if (N > blockDim.x) return;
    extern __shared__ float sdata[];
    const int tid = threadIdx.x;
    if (tid < N) sdata[tid] = s[tid];
    else       sdata[tid] = 0.0;

    __syncthreads();
    for (int s = blockDim.x / 2; s > 0; s >>= 1) {
        if (tid < s) {
            sdata[tid] += sdata[tid + s];
        }
        __syncthreads();
    }
    if (tid == 0)  s[0] = sdata[0];
}
//--------------------------------------------------------------------
float dscapr_GPU(const int n, const float x[], const float y[])
{
    cudaDeviceProp prop;
    cudaGetDeviceProperties (&prop, 0);
    const int blocksize = 4 * 64,
//           gridsize = ceil( N / static_cast<float>(blocksize)),
              gridsize = prop.multiProcessorCount,
              resBytes = gridsize * sizeof(float),
              sBytes   = blocksize * sizeof(float) ;
    dim3 dimBlock(blocksize);
    dim3 dimGrid(gridsize);
// allocate memory for temp. vector on device
    float *s_d;               // single number
    cudaMalloc((void **) &s_d, resBytes);


// call the kernel function with  dimGrid.x * dimBlock.x threads
    scalar <<< dimGrid, dimBlock, sBytes>>>(s_d, x, y, n);
//    power of 2 as number of treads per block
    const unsigned int oneBlock = 2 << (int)ceil(log(dimGrid.x + 0.0) / log(2.0));
    //cudaThreadSynchronize();
    add_1_block <<< 1, oneBlock, oneBlock *sizeof(float)>>>(s_d, dimGrid.x);
// copy data:  device --> host
    float sum;
    cudaMemcpy(&sum, s_d, sizeof(float), cudaMemcpyDeviceToHost);

    cudaFree(s_d);
    return sum;
}
