// scalar product: 4: use of cublasDdot 
#include <cassert>
#include <iostream>
#include <ctime>
using namespace std;
#include <cublas.h>
#include <cutil_inline.h>

//  main function on host
int main(void)
{
 double *x_h, *y_h; // host data
 double *x_d, *y_d; // device data
 double sum, tstart, tgpu;
 const int N = 14000000;
 const int LOOP = 100;
 
 if (cublasInit() != CUBLAS_STATUS_SUCCESS) {
        fprintf(stderr, "ERROR: cublasInit() failed!\n");
        exit(1);
    }
 
 cudaDeviceProp prop;
 cudaGetDeviceProperties (&prop, 0);
 cout<< endl << "We work on a "<< prop.name << " GPU  with " << 
       prop.multiProcessorCount << " Multiprocessors"<< endl;
           
 const int blocksize= 4*64, 
//           gridsize = ceil( N / static_cast<double>(blocksize)),
           gridsize = prop.multiProcessorCount;

 dim3 dimBlock(blocksize);
 dim3 dimGrid(gridsize); 

 cout << endl << gridsize << " x " << blocksize << " Threads\n";

 x_h = new double [N];
 y_h = new double [N];
// allocate memory on device
 cublasAlloc(N, sizeof(double), (void **) &x_d);
 cublasAlloc(N, sizeof(double), (void **) &y_d);

 double ss=0.0;
 for (int i=0; i<N; i++) 
   { x_h[i] = (i % 137)+1; y_h[i] = 1.0/x_h[i]; ss+=x_h[i]*y_h[i];}

// copy data:  host --> device
 cublasSetVector(N, sizeof(double), x_h, 1, x_d, 1);
 cublasSetVector(N, sizeof(double), y_h, 1, y_d, 1);
 sum = 0.0;
 cudaThreadSynchronize();
 tstart = clock();
 for (int k=0; k<LOOP; ++k) {
  sum += cublasDdot(N,x_d,1,y_d,1);
 }
 cudaThreadSynchronize();
 tgpu = (clock() - tstart)/CLOCKS_PER_SEC/LOOP; 
 
 cout << "GPU: " << tgpu << " sec.; " << 
   2e-9*N/tgpu << " GFLOPS;  " << 
   2.0*N*sizeof(double)/tgpu/1024/1024/1024 << " GiB/sec.\n ";
   
 if (static_cast<int>(sum/LOOP)!=N)
    {
    cout << " !!  W R O N G  result  " << endl;
    cout << sum << "  vs.  " << N << "(" << ss<< ")" << endl;
    }
 else
    cout << " correct  result  " << endl;

 delete [] x_h; delete [] y_h; 
 cublasFree(x_d); cublasFree(y_d);
 return 0;
}
