# Basic Defintions for using GNU-compiler suite sequentially
# requires setting of COMPILER=GCC_

#startmake as follows to avoid warnings caused by OpenMPI code
#  make 2>&1 | grep -v openmpi


#MPI_ROOT=/usr/bin/

CC	= ${MPI_ROOT}mpicc
CXX     = ${MPI_ROOT}mpicxx
F77	= ${MPI_ROOT}mpif77
LINKER  = ${CXX} 

MPIRUN  = ${MPI_ROOT}mpirun


WARNINGS = -Wall -pedantic -Wextra -Woverloaded-virtual -W -Wfloat-equal -Wshadow \
           -Wredundant-decls 
#           -Weffc++ -pedantic -Wunreachable-code -Wextra -Winline 
CXXFLAGS += -ffast-math -O3 -std=c++0x -funroll-all-loops -DNDEBUG -msse3 ${WARNINGS}
# -ftree-vectorizer-verbose=5
#          -ftree-vectorizer-verbose=2
# CFLAGS	= -ffast-math -O3 -DNDEBUG -msse3 -fopenmp -fdump-tree-vect-details
# CFLAGS	= -ffast-math -O3 -funroll-loops -DNDEBUG -msse3 -fopenmp -ftree-vectorizer-verbose=2
# #CFLAGS	= -ffast-math -O3 -DNDEBUG -msse3 -fopenmp
# FFLAGS	= -ffast-math -O3 -DNDEBUG -msse3 -fopenmp
# LFLAGS  = -ffast-math -O3 -DNDEBUG -msse3 -fopenmp

LINKFLAGS   += 
# -lcblas

# OpenMP
# CXXFLAGS += -fopenmp
# LINKFLAGS += -fopenmp

default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@
	@echo
	@echo "Start with :  $(MPIRUN) -np num_proc $(MPIFLAGS) $(PROGRAM)"
	@echo

clean:
	@rm -f ${PROGRAM} ${OBJECTS}

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: ${PROGRAM}
	${MPIRUN} -np 4 ./$^

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *
	
doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

# special: get rid of compiler warnings genereate by openmpi-files
.cpp.o:
# 	@$(CXX) -c $(CXXFLAGS) $< 2>/tmp/t.txt || grep -sv openmpi /tmp/t.txt
# 	|grep -sv openmpi

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
#    some tools
# Cache behaviour (CXXFLAGS += -g  tracks down to source lines)
cache: ${PROGRAM}
	valgrind --tool=callgrind --simulate-cache=yes ./$^
#	kcachegrind callgrind.out.<pid> &

# Check for wrong memory accesses, memory leaks, ...
# use smaller data sets
mem: ${PROGRAM}
	valgrind -v --leak-check=yes --tool=memcheck --undef-value-errors=yes --track-origins=yes --log-file=$^.addr.out --show-reachable=yes ${MPIRUN} -np 4 $^

#  Simple run time profiling of your code
#  CXXFLAGS += -g -pg
#  LINKFLAGS += -pg
prof: ${PROGRAM}
	./$^
	gprof -b ./$^ > gp.out
#	kprof -f gp.out -p gprof &
