# Basic Defintions for using INTEL compiler suite sequentially
# requires setting of COMPILER=ICC_NATIVE_

#  http://www.hpc.cineca.it/content/quick-guide-intel-mic-usage
#   for examples

# http://software.intel.com/en-us/articles/building-a-native-application-for-intel-xeon-phi-coprocessors
#  for: How to start a native application (runs exclusively on MIC)
##########################################################################################
# M. Luo, Mingzhe Li, A. Venkatesh, X. Lu, D.K. Panda - UPC on MIC: 
# Early Experiences with Native and Symmetric Modes, Conference on Partitioned Global Address Space Programming Models (PGAS), October 2013. 
#########################################################################################
# in the shell (~/.bashrc):
# export SINK_LD_LIBRARY_PATH=/opt/intel/composer_xe_2013_sp1.0.080/compiler/lib/mic:/opt/intel/composer_xe_2013_sp1.0.080/mkl/lib/mic

BINDIR=

# directory for special MIC programs (locate micnativeloadex)
MIC_BINDR=/opt/intel/mic/bin/

CC	= ${BINDIR}icc
CXX     = ${BINDIR}icpc
F77	= ${BINDIR}ifort
LINKER  = ${CXX} 


WARNINGS = -Wall -Weffc++ -Woverloaded-virtual -W -Wfloat-equal -Wshadow -wd2015,2012,11069
#  -Wunreachable-code  -Wredundant-decls -Winline

CXXFLAGS +=  -O3 -fargument-noalias  -DNDEBUG ${WARNINGS} -vec-report=3 -mkl

# OpenMP + MIC
CXXFLAGS += -openmp -mmic  -openmp_report2 
LINKFLAGS += -openmp -mmic

# use MKL by INTEL
# LINKFLAGS += -L${BINDIR}../composer_xe_2013.1.117/mkl/lib/intel64 -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -lpthread
LINKFLAGS +=  -mkl



default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@

clean:
	rm -f ${PROGRAM} ${OBJECTS}

clean_all: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
	${MIC_BINDR}micnativeloadex ${PWD}/${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
# #    some tools
# # Cache behaviour (CXXFLAGS += -g  tracks down to source lines)
# cache: ${PROGRAM}
# 	valgrind --tool=callgrind --simulate-cache=yes ./$^
# #	kcachegrind callgrind.out.<pid> &
# 
# # Check for wrong memory accesses, memory leaks, ...
# # use smaller data sets
# mem: ${PROGRAM}
# 	valgrind -v --leak-check=yes --tool=memcheck --undef-value-errors=yes --track-origins=yes --log-file=$^.addr.out --show-reachable=yes ./$^
# 
# #  Simple run time profiling of your code
# #  CXXFLAGS += -g -pg
# #  LINKFLAGS += -pg
# prof: ${PROGRAM}
# 	./$^
# 	gprof -b ./$^ > gp.out
# #	kprof -f gp.out -p gprof &
# 


mem: inspector
prof: amplifier
cache: amplifier

gap_par_report:
	${CXX}  -c -guide -parallel $(SOURCES) 2> gap.txt
	
# GUI for performance report
amplifier: ${PROGRAM}
	${BINDIR}../vtune_amplifier_xe_2013/bin64/amplxe-gui &

# GUI for Memory and Thread analyzer (race condition)
inspector: ${PROGRAM}
	${BINDIR}../inspector_xe_2013/bin64/inspxe-gui & 


info:
	 ps -a | grep coi
	 env |grep MIC
	 /opt/intel/mic/bin/micinfo
	 /opt/intel/mic/bin/micsmc -h
	 /opt/intel/mic/bin/micsmc -a
