# using MKL & OpenMP -> MKL link line advisor(http://software.intel.com/en-us/articles/intel-mkl-link-line-advisor)

CC	= ${BINDIR}icc
CXX     = ${BINDIR}icpc
F77	= ${BINDIR}ifort
LINKER  = ${CXX}

MKLROOT=/opt/intel/composer_xe_2013_sp1.0.080/mkl

MKL_LIBS=-Wl,--start-group $(MKLROOT)/lib/intel64/libmkl_intel_lp64.a $(MKLROOT)/lib/intel64/libmkl_core.a $(MKLROOT)/lib/intel64/libmkl_intel_thread.a -Wl,--end-group

MKL_MIC_LIBS=-Wl,--start-group $(MKLROOT)/lib/mic/libmkl_intel_lp64.a $(MKLROOT)/lib/mic/libmkl_core.a $(MKLROOT)/lib/mic/libmkl_intel_thread.a -Wl,--end-group

#WARNINGS = -Wall -Weffc++ -Woverloaded-virtual -Wfloat-equal -Wshadow -wd2015,2012,11069
#  -Wunreachable-code  -Wredundant-decls -Winline
#CXXFLAGS += -DNDEBUG ${WARNINGS}

CXXFLAGS += -O3 -vec-report3 -restrict -fno-alias -fargument-noalias -ansi-alias #-vec-report3 -parallel -par-report3 -guide
 # -ipo

# Compiler options are passed to MIC compiler automatically, use -offload-option,mic,compiler to pass additional options to compiler
# OpenMP + OFFLOAD
CXXFLAGS += -openmp -I$(MKLROOT)/include -offload-attribute-target=mic
#-openmp-report2

# Path to MIC libraries should be passed with -offload-option,mic,compiler
LINKFLAGS += -openmp $(MKL_LIBS) -lpthread -lm -offload-option,mic,compiler,"  $(MKL_MIC_LIBS)"


default:	${PROGRAM}

${PROGRAM}:	${OBJECTS}
	$(LINKER)  $^  ${LINKFLAGS} -o $@

clean:
	rm -f ${PROGRAM} ${OBJECTS} *MIC.o

clean_all:: clean
	@rm -f *_ *~ *.bak *.log *.out *.tar

run: clean ${PROGRAM}
	${PWD}/${PROGRAM}

# tar the current directory
MY_DIR = `basename ${PWD}`
tar: clean_all
	@echo "Tar the directory: " ${MY_DIR}
	@cd .. ;\
	tar cf ${MY_DIR}.tar ${MY_DIR} *default.mk ;\
	cd ${MY_DIR}
# 	tar cf `basename ${PWD}`.tar *

doc:
	doxygen Doxyfile

#########################################################################

.cpp.o:
	$(CXX) -c $(CXXFLAGS) $<

.c.o:
	$(CC) -c $(CFLAGS) $<

.f.o:
	$(F77) -c $(FFLAGS) $<

##################################################################################################
# #    some tools

MIC_info:
	 /usr/bin/micinfo
	 /usr/bin/micsmc -h
	 /usr/bin/micsmc -a

Host_info:
	 hostname
	 cat /etc/issue
	 /opt/intel/composer_xe_2013_sp1.0.080/mpirt/bin/intel64/cpuinfo

# Debugger:
# The GNU* Project Debugger (GDB)
# http://software.intel.com/sites/default/files/article/256677/intel-mic-gdb.pdf
gdb: ${PROGRAM}
	gdb	${file} ${PWD}/${PROGRAM}
# type "start" to start debugging and "Quit" to exit

# Performance analyzers:
# Intel trace analyzer and collector(ITAC)
# Intel VTune Amplifier XE
# Scalasca

prof: amplifier
cache: amplifier

# GUI for performance report
amplifier: ${PROGRAM}
	/opt/intel/vtune_amplifier_xe_2013/bin64/amplxe-gui &
