#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER 
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include <omp.h>                // OpenMP

#include "mylib.h"


void matmul(double b[], const double A[], const unsigned int M,
	    const unsigned int N, const double x[])
{
unsigned int i,j;
#pragma omp parallel for private(i,j)  shared(A,x,b) schedule(static)
  for ( i=0; i<M; ++i)
  {
    double s(0.0);
    for (j=0; j<N; ++j)
    {
      s += A[i*N+j] * x[j];
    }
    b[i] = s;
  }
}

void matmul_blas(double b[], const double A[], const unsigned int M,
	    const unsigned int N, const double x[])
{
  cblas_dgemv(CblasRowMajor,CblasNoTrans,M,N,1.0,A,N, x,1, 0.0,b,1);
}
