//	general header for all functions in directory

#ifndef HEADER_DEXX
#define HEADER_DEXX
#include <mpi.h>
/**
 * Sends data of float-vector @p xin to the process with rank @p to .
 * @param[in] to 	rank of process in communicator @p icomm to send the message
 * @param[in] nin 	number of elements in vector @p xin
 * @param[in] xin	vector
 * @param[in] icomm	communicator to use
*/

void Send_ProcD(const int to, const int nin, const float xin[], const MPI_Comm icomm);

/**
 * Receives data from process with rank @p from into a float-vector @p xout .
 * @param[in] from 	rank of process in communicator @p icomm to receive the message from
 * @param[out] nout 	number of received elements stored in vector @p xout
 * @param[out] xout	vector
 * @param[in] maxbuf	maximal number of elements of  @p xout
 * @param[in] icomm	communicator to use
*/
void Recv_ProcD(const int from, int& nout, float xout[], const int maxbuf,
	const MPI_Comm icomm);

/**
 * Exchanges vectors between my process and  the process with rank @yourid .
 * @param[in] yourid 	rank of process in communicator @p icomm to send the message
 * @param[in] nin 	number of elements in vector @p xin
 * @param[in] xin	my vector
 * @param[out] nout 	number of received elements stored in vector @p xout
 * @param[out] xout	vector
 * @param[in] maxbuf	maximal number of elements of  @p xout
 * @param[in] icomm	communicator to use
*/
void ExchangeD(const int yourid, const int nin, const float xin[],
	int& nout, float xout[], const int maxbuf, const MPI_Comm icomm);

#endif
