#ifndef JACSOLVE_FILE
#define JACSOLVE_FILE
#include <mpi.h>

/**
 * Solves linear system of equations  K @p u = @p f  via the Jacobi iteration.
 * We use a distributed symmetric  CSR matrix K(@p sk,@p id, @p ik) and initial guess of the
 * solution is set to 0.
 * @param[in] nnode	local number of nodes and number of rows
 * @param[in] nx	local number intervals in x-direction
 * @param[in] ny	local number intervals in x-direction
 * @param[in] neigh	neighbors in parallel
 * @param[in] color	my color in parallel
 * @param[in] icomm MPI communicator
 * @param[in] sk	vector non-zero entries of CSR matrix
 * @param[in] id	index vector containing the last entry in a CSR row
 * @param[in] ik	column index vector of CSR matrix
 * @param[in] f		distributed local vector storing the right hand side
 * @param[out] u	accumulated local vector storing the solution.
*/

void JacobiSolve(const int nnode,const int nx, const int ny,
 	const int neigh[], const int color,	const MPI_Comm icomm,
                 const int id[], const int ik[], const float sk[],
                 const float f[], float u[] );



#endif

