#include <iostream>
#include <cstdlib>
using namespace std;
#include <mpi.h>

const int TAG1 = 1;
const int TAG2 = 2;

void Send_ProcD(int to, int nin, const double xin[], const MPI_Comm icomm)
{
//   cout << icomm.Get_rank() << "  -->  " << to << "  (length " << nin << ")\n";

  MPI_Send(&nin, 1, MPI_INT, to, TAG1, icomm);

  if (nin > 0)
     MPI_Send(const_cast<double*>(xin), nin, MPI_DOUBLE, to, TAG2, icomm);
  return;
}



void Recv_ProcD(int from, int& nout, double xout[], const int maxbuf,
	const MPI_Comm icomm)
{
//   cout << icomm.Get_rank() << "  <--  " << from << "\n";
  MPI_Status stat;
  stat.MPI_ERROR=0;              // M U S T   be initialized!!

  MPI_Recv(&nout, 1, MPI_INT, from, TAG1, icomm, &stat);
  int info=stat.MPI_ERROR;            // C++: int info=stat.Get_error();
  
  //int myrank;
  //MPI_Comm_rank(icomm,&myrank);
  //cout <<myrank <<  " :: nout = " << nout << " info: " << info<< endl;

  if (info!=MPI_SUCCESS)
  { cout << "\n Recv_ProcD : " << info << endl; exit(-3);}

  if (nout>maxbuf)
  { cout << "\n Recv_ProcD : maxbuf (= " << maxbuf << ") is too small vs. "
         << nout << endl; exit(-3); }

  if (nout > 0)
  {
    MPI_Recv(xout, nout, MPI_DOUBLE, from, TAG2, icomm, &stat);
  }

  return;
}



void ExchangeD(const int yourid, const int nin, const double xin[],
	int& nout, double xout[], const int maxbuf, const MPI_Comm icomm)
{
  int myrank;
  MPI_Comm_rank(icomm,&myrank);
      
  if (yourid < 0) { nout = -1; return; }
//   cout << myrank << "  :  " << yourid << endl;

  if (myrank < yourid)
     { Send_ProcD(yourid,nin,xin,icomm);
       Recv_ProcD(yourid,nout,xout,maxbuf,icomm); }
    else
     { Recv_ProcD(yourid,nout,xout,maxbuf,icomm);
       Send_ProcD(yourid,nin,xin,icomm); }

//   cout << icomm.Get_rank() << "  #  " << yourid << "    nout= " << nout << endl;
  return;
}
