#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER 
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
                                   //  _MKL_BLAS_H_     is defined
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include "mylib.h"

void matmat(const unsigned int M, const unsigned int N, const unsigned int L,
            double C[], const double A[], const double B[])
{
#pragma parallel
  for (unsigned int i=0; i<M; ++i)
  {
    for (unsigned int j=0; j<N; ++j)
    {
      double s(0.0);
      for (unsigned int k=0; k<L; ++k)
      {
         s += A[i*L+k] * B[k*N+j];
//          s += A[i*L+k] * B[j*L+k];             // columnwise access 4 times faster
      }
      C[i*N+j] = s;
    }
  }
}

// seen in code from Anyyeva
void matmat_2(const unsigned int M, const unsigned int N, const unsigned int L,
            double C[], const double A[], const double B[])
{
#pragma loop count min(32)
#pragma parallel
  for (unsigned int i=0; i<M; ++i)
  {
    for (unsigned int j=0; j<N; ++j)  C[i*N+j] = 0.0;
    for (unsigned int k=0; k<L; ++k)
    {
      const double & Aik = A[i*L+k];
      for (unsigned int j=0; j<N; ++j)
      {
         C[i*N+j] +=  Aik * B[k*N+j];
      }
    }
  }
}

// seen in code from Anyyeva
void matmat_3(const unsigned int M, const unsigned int N, const unsigned int L,
             double* __restrict C, const double * const __restrict A, const double * const __restrict B)
{
#pragma parallel
  for (unsigned int i=0; i<M; ++i)
  {
#pragma loop count min(1024)
    for (unsigned int j=i*N; j<(i+1)*N; ++j)  C[j] = 0.0;
    for (unsigned int k=0; k<L; ++k)
    {
      const double & Aik = A[i*L+k];
      double s(0.0);
#pragma loop count min(256)
      for (unsigned int j=0; j<N; ++j)
      {
         C[i*N+j] +=  Aik * B[k*N+j];
      }
    }
  }
}

/** 	Performs dense matrix-matrix product C    = A    * B
                                              MxN    MxL    LXN
	matrices are rowise stored
*/
void matmat_blas(const int M, const int N, const int L,
            double C[], const double A[], const double B[])
{
  cblas_dgemm (CblasRowMajor, CblasNoTrans, CblasNoTrans, M, N, L, 1.0, A, L, B, N, 0.0, C, N);
}

