#include <iostream>
#include <sstream>

#include <cstdlib>
#include <cstring>
#include <cmath>
#include <ctime>
#include "mylib.h"
using namespace std;

int main(int argc, char **argv)
{
  const int NLOOPS=20;          // chose a value such that the benchmark runs at least 10 sec.
  const unsigned int P_DEGREE=64;// Degree of Polynom.
  unsigned int N=10000000;
//##########################################################################
//   Read Paramater from command line
  cout << "Checking arguments: N= " << argc << endl;
  for(int i=0;i<argc;i++)
     {
      cout << " arg[" << i << "] = " << argv[i] << endl;
      if (strncmp(argv[i],"-n",2)==0)
       {
	N = atoi(argv[i+1]);
       }
       else
       {
	 cout << "Correct call: " << argv[i] << " N=  <number>\n";
       }
   }
  cout << "\nN = " << N << endl;

//  Memory allocation
  cout << "Memory allocation\n";

  double *x,*y,*a;
  x = new double [N];
  y = new double [N];
  a = new double [P_DEGREE+1];

  cout.precision(2);
  cout << (2.0*N+P_DEGREE+1)*sizeof(x[0])/1024/1024/1024 << " GByte Memory allocated\n";
  cout.precision(6);

//##########################################################################
//  Data initialization
//  Special:  x_i = i+1;  y_i = 1/x_i  ==> <x,y> == N
  for (unsigned int i=0; i<N; ++i)
  {
    x[i] = 1.0/( (i+1)%29 );
  }

  for (unsigned int i=0; i<=P_DEGREE; ++i)
  {
    a[i] = 1.23*sin(i*M_PI/(P_DEGREE+1));
  }

//##########################################################################
  cout << "\nStart Benchmarking\n";

  double tstart, t1;  // timer

// // Do calculation
//   tstart = clock();       // start timer
//
//   for (int i=0; i<NLOOPS; ++i)
//   {
//      func_poly_1(P_DEGREE, a, N, x, y);
//   }
//
//   t1 = clock()-tstart;    // stop timer
//   t1 /= CLOCKS_PER_SEC;   // now, t1 in seconds
//   t1 /= NLOOPS;           // divide by number of function calls
//
// //##########################################################################
// // Timings  and Performance
//  cout << endl << " Horner " << endl;
//  cout.precision(2);
//  cout << "Timing in sec. : " << t1 << endl;
//   // FLOPS depends on your implementation
// cout << "GFLOPS         : " << (3.0*P_DEGREE*N)/t1/1024/1024/1024 << endl;
//  // Transfer depends on your implementation
//  cout << "GByte/s        : " << 2.0*N/t1/1024/1024/1024*sizeof(x[0]) << endl;
//
//  //##########################################################################
//   cout << "\nStart Benchmarking\n";
//
// // Do calculation
//   tstart = clock();       // start timer
//
//   for (int i=0; i<NLOOPS; ++i)
//   {
//      func_poly_2(P_DEGREE, a, N, x, y);
//   }
//
//   t1 = clock()-tstart;    // stop timer
//   t1 /= CLOCKS_PER_SEC;   // now, t1 in seconds
//   t1 /= NLOOPS;           // divide by number of function calls
//
// //##########################################################################
// // Timings  and Performance
//  cout << endl << " mult-poly " << endl;
//  cout.precision(2);
//  cout << "Timing in sec. : " << t1 << endl;
//   // FLOPS depends on your implementation
// cout << "GFLOPS         : " << (3.0*P_DEGREE*N)/t1/1024/1024/1024 << endl;
//  // Transfer depends on your implementation
//  cout << "GByte/s        : " << 2.0*N/t1/1024/1024/1024*sizeof(x[0]) << endl;

 //##########################################################################
  cout << "\nStart Benchmarking\n";

// Do calculation
  tstart = clock();       // start timer

  for (int i=0; i<NLOOPS; ++i)
  {
     func_poly_3_unroll(P_DEGREE, a, N, x, y);
  }

  t1 = clock()-tstart;    // stop timer
  t1 /= CLOCKS_PER_SEC;   // now, t1 in seconds
  t1 /= NLOOPS;           // divide by number of function calls

//##########################################################################
// Timings  and Performance
 cout << endl << " mult-poly + unroll " << endl;
 cout.precision(2);
 cout << "Timing in sec. : " << t1 << endl;
  // FLOPS depends on your implementation
cout << "GFLOPS         : " << (3.0*P_DEGREE*N)/t1/1024/1024/1024 << endl;
 // Transfer depends on your implementation
 cout << "GByte/s        : " << 2.0*N/t1/1024/1024/1024*sizeof(x[0]) << endl;

//##########################################################################
//  Free allocated memory
  delete [] a; delete [] y; delete [] x;

  return 0;
}
