#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER 
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
                                   //  _MKL_BLAS_H_     is defined
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include "mylib.h"


double scalar(const unsigned int N, const double x[], const double y[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}

double norm(const unsigned int N, const double x[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*x[i];
 }
 return sqrt(sum);
}


double poly(const unsigned int p, const double coeff[], const double x)
{
  double sum = coeff[0],
         xp  = 1.0;    // x^0

  for (int k=1; k<=p; ++k)
  {
    xp  *= x;             // x^k
    sum += coeff[k]*xp;   // + a_k*x^k
  }

  return sum;
}

double horner(const unsigned int p, const double coeff[], const double x)
{
  double sum = coeff[p];

  for (int k=p-1; k>=0; --k)
  {
    sum = x*sum + coeff[k];        // x*()+a_k
  }

  return sum;
}


void func_poly_1(const unsigned int p, const double coeff[],
		 const unsigned int N, const double x[], double y[])
{
#pragma parallel
#pragma loop count min(16)
  for (unsigned int i=0; i<N; ++i)
  {
    y[i] = horner(p,coeff,x[i]);
  }
  return;
}


void func_poly_2(const unsigned int p, const double coeff[],
		 const unsigned int N, const double x[], double y[])
{
#pragma parallel
#pragma loop count min(16)
  for (unsigned int i=0; i<N; ++i)
  {
    double sum = coeff[0],
           xp  = 1.0;     // x^0
    const double  xi  = x[i];

    for (int k=1; k<=p; ++k)
    {
      xp  *= xi;             // x^k
      sum += coeff[k]*xp;   // + a_k*x^k
    }
    y[i] = sum;
  }

  return;
}

void func_poly_2_unroll(const unsigned int p, const double coeff[],
		 const unsigned int N, const double x[], double y[])
{
#pragma parallel
  for (unsigned int i=0; i<N; ++i)
  {
    double sum = coeff[0],
           xp1  = x[i],        // x^0
           xp2  = xp1*xp1, xp3 = xp2*xp1, xp4 = xp2*xp2;
    const double  xi  = xp1, x4 = xp4;

    for (int k=1; k<=(p/4)*4; k+=4)
    {
      sum += coeff[k]*xp1+coeff[k+1]*xp2+coeff[k+2]*xp3+coeff[k+3]*xp4;
      xp1 *= x4;
      xp2 *= x4;
      xp3 *= x4;
      xp4 *= x4;
    }
    for (int k=(p/4)*4+1; k<=p; ++k)
    {
      sum += coeff[k]*xp1;   // + a_k*x^k
      xp1 *= xi;
    }
    y[i] = sum;
  }

  return;
}
