#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER 
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include "mylib.h"


double scalar(const unsigned int N, const double x[], const double y[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*y[i];
//     sum += exp(x[i])*log(y[i]);
 }
 return sum;
}

/*
double scalar_p(const unsigned int N, const double x[], const double y[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; i+=8)
 {
    sum += x[i  ]*y[i  ]+x[i+1]*y[i+1]+x[i+2]*y[i+2]+x[i+3]*y[i+3]
          +x[i+4]*y[i+4]+x[i+5]*y[i+5]+x[i+6]*y[i+6]+x[i+7]*y[i+7];

 }
 return sum;
}
*/

double scalar_p(const unsigned int N, const double x[], const double y[])
{
#ifndef __PGI
// my Intel E7300 has cache lines of 64 byte size
 __builtin_prefetch (&x[0], 0, 0);
 __builtin_prefetch (&y[0], 0, 0);
#endif
 double sum0 = 0.0, sum1 = 0.0, sum2 = 0.0, sum3 = 0.0;
 for (unsigned int i=0; i<(N/8)*8; i+=8)
 {
#ifndef __PGI
//    __builtin_prefetch (&x[i+4], 0, 0);
//    __builtin_prefetch (&y[i+4], 0, 0);
    __builtin_prefetch (&x[i+8], 0, 0);
    __builtin_prefetch (&y[i+8], 0, 0);
#endif
    sum0 += x[i  ]*y[i  ] + x[i+4]*y[i+4];
    sum1 += x[i+1]*y[i+1] + x[i+5]*y[i+5];
    sum2 += x[i+2]*y[i+2] + x[i+6]*y[i+6];
    sum3 += x[i+3]*y[i+3] + x[i+7]*y[i+7];
 }
 double sum = sum0+sum1+sum2+sum3;

 for (unsigned int i=(N/8)*8; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}

double scalar_p2(const unsigned int N, const double x[], const double y[])
{
#ifndef __PGI
// my Intel E7300 has cache lines of 64 byte size
 __builtin_prefetch (&x[0], 0, 0);
 __builtin_prefetch (&y[0], 0, 0);
#endif
 double ss[4]={0.0,0.0,0.0,0.0};
 for (unsigned int i=0; i<(N/8)*8; i+=8)
 {
#ifndef __PGI
//    __builtin_prefetch (&x[i+4], 0, 0);
//    __builtin_prefetch (&y[i+4], 0, 0);
//     __builtin_prefetch (&x[i+8], 0, 0);
//     __builtin_prefetch (&y[i+8], 0, 0);
#endif
    ss[0] += x[i  ]*y[i  ] + x[i+4]*y[i+4];
    ss[1] += x[i+1]*y[i+1] + x[i+5]*y[i+5];
    ss[2] += x[i+2]*y[i+2] + x[i+6]*y[i+6];
    ss[3] += x[i+3]*y[i+3] + x[i+7]*y[i+7];
 }
 double sum = ss[0]+ss[1]+ss[2]+ss[3];

 for (unsigned int i=(N/8)*8; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}


double scalar_blas(const unsigned int N, const double x[], const double y[])
{
 return cblas_ddot(N,x,1,y,1);
}




double norm(const unsigned int N, const double x[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*x[i];
 }
 return sqrt(sum);
}

