#include <cmath>
using namespace std;

#ifdef __INTEL_COMPILER
  #pragma message(" ##########  Use of MKL  ###############")
  #include <mkl.h>                 // MKL
#else
  #pragma message(" ##########  Use of CBLAS  ###############")
extern "C"
{
  #include <cblas.h>               // cBLAS Library
}
#endif

#include <omp.h>                // OpenMP

#include "mylib.h"


double scalar(const unsigned int N, const double x[], const double y[])
{
 double sum = 0.0;
 unsigned int i;
#pragma omp parallel for private(i) shared(x,y) schedule(static) reduction(+:sum)
 for (i=0; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}

double scalar_p(const unsigned int N, const double x[], const double y[])
{
 double sum0 = 0.0, sum1 = 0.0, sum2 = 0.0, sum3 = 0.0;
 unsigned int i;
#pragma omp parallel for private(i) shared(x,y) schedule(static) reduction(+:sum0,sum1,sum2,sum3)
 for (i=0; i<(N/8)*8; i+=8)
 {
    sum0 += x[i  ]*y[i  ] + x[i+4]*y[i+4];
    sum1 += x[i+1]*y[i+1] + x[i+5]*y[i+5];
    sum2 += x[i+2]*y[i+2] + x[i+6]*y[i+6];
    sum3 += x[i+3]*y[i+3] + x[i+7]*y[i+7];
 }
 double sum = sum0+sum1+sum2+sum3;

 for (i=(N/8)*8; i<N; ++i)
 {
    sum += x[i]*y[i];
 }
 return sum;
}


double scalar_blas(const unsigned int N, const double x[], const double y[])
{
 return cblas_ddot(N,x,1,y,1);
}




double norm(const unsigned int N, const double x[])
{
 double sum = 0.0;
 for (unsigned int i=0; i<N; ++i)
 {
    sum += x[i]*x[i];
 }
 return sqrt(sum);
}

